@@ -1,22 +1,12 @@
 # encoding: utf-8
-"""
-Context managers for temporarily updating dictionaries.
-
-Authors:
-
-* Bradley Froehle
+"""Miscellaneous context managers.
 """
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2012  The IPython Development Team
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
+import warnings
+
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
-#-----------------------------------------------------------------------------
-# Code
-#-----------------------------------------------------------------------------
 
 class preserve_keys(object):
     """Preserve a set of keys in a dictionary.
