@@ -1,4 +1,3 @@
-# encoding: utf-8
 """
 Utilities for working with strings and text.
 
@@ -17,7 +16,25 @@ import warnings
 from string import Formatter
 from pathlib import Path
 
-from typing import List, Dict, Tuple
+from typing import (
+    List,
+    Dict,
+    Tuple,
+    Optional,
+    cast,
+    Sequence,
+    Mapping,
+    Any,
+    Union,
+    Callable,
+    Iterator,
+    TypeVar,
+)
+
+if sys.version_info < (3, 12):
+    from typing_extensions import Self
+else:
+    from typing import Self
 
 
 class LSString(str):
@@ -36,7 +53,11 @@ class LSString(str):
     Such strings are very useful to efficiently interact with the shell, which
     typically only understands whitespace-separated options for commands."""
 
-    def get_list(self):
+    __list: List[str]
+    __spstr: str
+    __paths: List[Path]
+
+    def get_list(self) -> List[str]:
         try:
             return self.__list
         except AttributeError:
@@ -45,7 +66,7 @@ class LSString(str):
 
     l = list = property(get_list)
 
-    def get_spstr(self):
+    def get_spstr(self) -> str:
         try:
             return self.__spstr
         except AttributeError:
@@ -54,12 +75,12 @@ class LSString(str):
 
     s = spstr = property(get_spstr)
 
-    def get_nlstr(self):
+    def get_nlstr(self) -> Self:
         return self
 
     n = nlstr = property(get_nlstr)
 
-    def get_paths(self):
+    def get_paths(self) -> List[Path]:
         try:
             return self.__paths
         except AttributeError:
@@ -74,8 +95,8 @@ class LSString(str):
 
 # def print_lsstring(arg):
 #     """ Prettier (non-repr-like) and more informative printer for LSString """
-#     print "LSString (.p, .n, .l, .s available). Value:"
-#     print arg
+#     print("LSString (.p, .n, .l, .s available). Value:")
+#     print(arg)
 #
 #
 # print_lsstring = result_display.register(LSString)(print_lsstring)
@@ -94,12 +115,16 @@ class SList(list):
     Any values which require transformations are computed only once and
     cached."""
 
-    def get_list(self):
+    __spstr: str
+    __nlstr: str
+    __paths: List[Path]
+
+    def get_list(self) -> Self:
         return self
 
     l = list = property(get_list)
 
-    def get_spstr(self):
+    def get_spstr(self) -> str:
         try:
             return self.__spstr
         except AttributeError:
@@ -108,7 +133,7 @@ class SList(list):
 
     s = spstr = property(get_spstr)
 
-    def get_nlstr(self):
+    def get_nlstr(self) -> str:
         try:
             return self.__nlstr
         except AttributeError:
@@ -117,7 +142,7 @@ class SList(list):
 
     n = nlstr = property(get_nlstr)
 
-    def get_paths(self):
+    def get_paths(self) -> List[Path]:
         try:
             return self.__paths
         except AttributeError:
@@ -126,8 +151,13 @@ class SList(list):
 
     p = paths = property(get_paths)
 
-    def grep(self, pattern, prune = False, field = None):
-        """ Return all strings matching 'pattern' (a regex or callable)
+    def grep(
+        self,
+        pattern: Union[str, Callable[[Any], re.Match[str] | None]],
+        prune: bool = False,
+        field: Optional[int] = None,
+    ) -> Self:
+        """Return all strings matching 'pattern' (a regex or callable)
 
         This is case-insensitive. If prune is true, return all items
         NOT matching the pattern.
@@ -142,7 +172,7 @@ class SList(list):
             a.grep('chm', field=-1)
         """
 
-        def match_target(s):
+        def match_target(s: str) -> str:
             if field is None:
                 return s
             parts = s.split()
@@ -157,12 +187,12 @@ class SList(list):
         else:
             pred = pattern
         if not prune:
-            return SList([el for el in self if pred(match_target(el))])
+            return type(self)([el for el in self if pred(match_target(el))])
         else:
-            return SList([el for el in self if not pred(match_target(el))])
+            return type(self)([el for el in self if not pred(match_target(el))])
 
-    def fields(self, *fields):
-        """ Collect whitespace-separated fields from string list
+    def fields(self, *fields: List[str]) -> List[List[str]]:
+        """Collect whitespace-separated fields from string list
 
         Allows quick awk-like usage of string lists.
 
@@ -197,8 +227,12 @@ class SList(list):
 
         return res
 
-    def sort(self,field= None,  nums = False):
-        """ sort by specified fields (see fields())
+    def sort(  # type:ignore[override]
+        self,
+        field: Optional[List[str]] = None,
+        nums: bool = False,
+    ) -> Self:
+        """sort by specified fields (see fields())
 
         Example::
 
@@ -224,7 +258,7 @@ class SList(list):
 
 
         dsu.sort()
-        return SList([t[1] for t in dsu])
+        return type(self)([t[1] for t in dsu])
 
 
 # FIXME: We need to reimplement type specific displayhook and then add this
@@ -233,7 +267,7 @@ class SList(list):
 
 # def print_slist(arg):
 #     """ Prettier (non-repr-like) and more informative printer for SList """
-#     print "SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):"
+#     print("SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):")
 #     if hasattr(arg,  'hideonce') and arg.hideonce:
 #         arg.hideonce = False
 #         return
@@ -243,7 +277,7 @@ class SList(list):
 # print_slist = result_display.register(SList)(print_slist)
 
 
-def indent(instr,nspaces=4, ntabs=0, flatten=False):
+def indent(instr: str, nspaces: int = 4, ntabs: int = 0, flatten: bool = False) -> str:
     """Indent a string a given number of spaces or tabstops.
 
     indent(str,nspaces=4,ntabs=0) -> indent str by ntabs+nspaces.
@@ -263,7 +297,7 @@ def indent(instr,nspaces=4, ntabs=0, flatten=False):
 
     Returns
     -------
-    str|unicode : string indented by ntabs and nspaces.
+    str : string indented by ntabs and nspaces.
 
     """
     if instr is None:
@@ -280,7 +314,7 @@ def indent(instr,nspaces=4, ntabs=0, flatten=False):
         return outstr
 
 
-def list_strings(arg):
+def list_strings(arg: Union[str, List[str]]) -> List[str]:
     """Always return a list of strings, given a string or list of strings
     as input.
 
@@ -304,7 +338,7 @@ def list_strings(arg):
         return arg
 
 
-def marquee(txt='',width=78,mark='*'):
+def marquee(txt: str = "", width: int = 78, mark: str = "*") -> str:
     """Return the input string centered in a 'marquee'.
 
     Examples
@@ -331,11 +365,12 @@ def marquee(txt='',width=78,mark='*'):
 
 ini_spaces_re = re.compile(r'^(\s+)')
 
-def num_ini_spaces(strng):
+
+def num_ini_spaces(strng: str) -> int:
     """Return the number of initial spaces in a string"""
     warnings.warn(
         "`num_ini_spaces` is Pending Deprecation since IPython 8.17."
-        "It is considered fro removal in in future version. "
+        "It is considered for removal in in future version. "
         "Please open an issue if you believe it should be kept.",
         stacklevel=2,
         category=PendingDeprecationWarning,
@@ -347,7 +382,7 @@ def num_ini_spaces(strng):
         return 0
 
 
-def format_screen(strng):
+def format_screen(strng: str) -> str:
     """Format a string for screen printing.
 
     This removes some latex-type format codes."""
@@ -384,7 +419,7 @@ def dedent(text: str) -> str:
     return '\n'.join([first, rest])
 
 
-def wrap_paragraphs(text, ncols=80):
+def wrap_paragraphs(text: str, ncols: int = 80) -> List[str]:
     """Wrap multiple paragraphs to fit a specified width.
 
     This is equivalent to textwrap.wrap, but with support for multiple
@@ -396,7 +431,7 @@ def wrap_paragraphs(text, ncols=80):
     """
     warnings.warn(
         "`wrap_paragraphs` is Pending Deprecation since IPython 8.17."
-        "It is considered fro removal in in future version. "
+        "It is considered for removal in in future version. "
         "Please open an issue if you believe it should be kept.",
         stacklevel=2,
         category=PendingDeprecationWarning,
@@ -416,7 +451,7 @@ def wrap_paragraphs(text, ncols=80):
     return out_ps
 
 
-def strip_email_quotes(text):
+def strip_email_quotes(text: str) -> str:
     """Strip leading email quotation characters ('>').
 
     Removes any combination of leading '>' interspersed with whitespace that
@@ -466,7 +501,7 @@ def strip_email_quotes(text):
     return text
 
 
-def strip_ansi(source):
+def strip_ansi(source: str) -> str:
     """
     Remove ansi escape codes from text.
 
@@ -477,7 +512,7 @@ def strip_ansi(source):
     """
     warnings.warn(
         "`strip_ansi` is Pending Deprecation since IPython 8.17."
-        "It is considered fro removal in in future version. "
+        "It is considered for removal in in future version. "
         "Please open an issue if you believe it should be kept.",
         stacklevel=2,
         category=PendingDeprecationWarning,
@@ -507,7 +542,8 @@ class EvalFormatter(Formatter):
         In [3]: f.format("{greeting[slice(2,4)]}", greeting="Hello")
         Out[3]: 'll'
     """
-    def get_field(self, name, args, kwargs):
+
+    def get_field(self, name: str, args: Any, kwargs: Any) -> Tuple[Any, str]:
         v = eval(name, kwargs)
         return v, name
 
@@ -540,11 +576,14 @@ class FullEvalFormatter(Formatter):
     """
     # copied from Formatter._vformat with minor changes to allow eval
     # and replace the format_spec code with slicing
-    def vformat(self, format_string:str, args, kwargs)->str:
+    def vformat(
+        self, format_string: str, args: Sequence[Any], kwargs: Mapping[str, Any]
+    ) -> str:
         result = []
-        for literal_text, field_name, format_spec, conversion in \
-                self.parse(format_string):
-
+        conversion: Optional[str]
+        for literal_text, field_name, format_spec, conversion in self.parse(
+            format_string
+        ):
             # output the literal text
             if literal_text:
                 result.append(literal_text)
@@ -560,10 +599,11 @@ class FullEvalFormatter(Formatter):
 
                 # eval the contents of the field for the object
                 # to be formatted
-                obj = eval(field_name, kwargs)
+                obj = eval(field_name, dict(kwargs))
 
                 # do any conversion on the resulting object
-                obj = self.convert_field(obj, conversion)
+                # type issue in typeshed, fined in https://github.com/python/typeshed/pull/11377
+                obj = self.convert_field(obj, conversion)  # type: ignore[arg-type]
 
                 # format the object and append to the result
                 result.append(self.format_field(obj, ''))
@@ -590,11 +630,15 @@ class DollarFormatter(FullEvalFormatter):
         In [4]: f.format('$a or {b}', a=1, b=2)
         Out[4]: '1 or 2'
     """
-    _dollar_pattern_ignore_single_quote = re.compile(r"(.*?)\$(\$?[\w\.]+)(?=([^']*'[^']*')*[^']*$)")
-    def parse(self, fmt_string):
-        for literal_txt, field_name, format_spec, conversion \
-                    in Formatter.parse(self, fmt_string):
-            
+
+    _dollar_pattern_ignore_single_quote = re.compile(
+        r"(.*?)\$(\$?[\w\.]+)(?=([^']*'[^']*')*[^']*$)"
+    )
+
+    def parse(self, fmt_string: str) -> Iterator[Tuple[Any, Any, Any, Any]]:  # type: ignore
+        for literal_txt, field_name, format_spec, conversion in Formatter.parse(
+            self, fmt_string
+        ):
             # Find $foo patterns in the literal text.
             continue_from = 0
             txt = ""
@@ -611,14 +655,17 @@ class DollarFormatter(FullEvalFormatter):
             # Re-yield the {foo} style pattern
             yield (txt + literal_txt[continue_from:], field_name, format_spec, conversion)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "<DollarFormatter>"
 
 #-----------------------------------------------------------------------------
 # Utils to columnize a list of string
 #-----------------------------------------------------------------------------
 
-def _col_chunks(l, max_rows, row_first=False):
+
+def _col_chunks(
+    l: List[int], max_rows: int, row_first: bool = False
+) -> Iterator[List[int]]:
     """Yield successive max_rows-sized column chunks from l."""
     if row_first:
         ncols = (len(l) // max_rows) + (len(l) % max_rows > 0)
@@ -629,7 +676,9 @@ def _col_chunks(l, max_rows, row_first=False):
             yield l[i:(i + max_rows)]
 
 
-def _find_optimal(rlist, row_first: bool, separator_size: int, displaywidth: int):
+def _find_optimal(
+    rlist: List[int], row_first: bool, separator_size: int, displaywidth: int
+) -> Dict[str, Any]:
     """Calculate optimal info to columnize a list of string"""
     for max_rows in range(1, len(rlist) + 1):
         col_widths = list(map(max, _col_chunks(rlist, max_rows, row_first)))
@@ -644,7 +693,10 @@ def _find_optimal(rlist, row_first: bool, separator_size: int, displaywidth: int
             }
 
 
-def _get_or_default(mylist, i, default=None):
+T = TypeVar("T")
+
+
+def _get_or_default(mylist: List[T], i: int, default: T) -> T:
     """return list item number, or default if don't exist"""
     if i >= len(mylist):
         return default
@@ -653,7 +705,12 @@ def _get_or_default(mylist, i, default=None):
 
 
 def compute_item_matrix(
-    items, row_first: bool = False, empty=None, *, separator_size=2, displaywidth=80
+    items: List[str],
+    row_first: bool = False,
+    empty: Optional[str] = None,
+    *,
+    separator_size: int = 2,
+    displaywidth: int = 80,
 ) -> Tuple[List[List[int]], Dict[str, int]]:
     """Returns a nested list, and info to columnize items
 
@@ -704,25 +761,53 @@ def compute_item_matrix(
     """
     warnings.warn(
         "`compute_item_matrix` is Pending Deprecation since IPython 8.17."
-        "It is considered fro removal in in future version. "
+        "It is considered for removal in in future version. "
         "Please open an issue if you believe it should be kept.",
         stacklevel=2,
         category=PendingDeprecationWarning,
     )
     info = _find_optimal(
-        list(map(len, items)),
+        list(map(len, items)),  # type: ignore[arg-type]
         row_first,
         separator_size=separator_size,
         displaywidth=displaywidth,
     )
     nrow, ncol = info["max_rows"], info["num_columns"]
     if row_first:
-        return ([[_get_or_default(items, r * ncol + c, default=empty) for c in range(ncol)] for r in range(nrow)], info)
+        return (
+            [
+                [
+                    _get_or_default(
+                        items, r * ncol + c, default=empty
+                    )  # type:ignore[misc]
+                    for c in range(ncol)
+                ]
+                for r in range(nrow)
+            ],
+            info,
+        )
     else:
-        return ([[_get_or_default(items, c * nrow + r, default=empty) for c in range(ncol)] for r in range(nrow)], info)
+        return (
+            [
+                [
+                    _get_or_default(
+                        items, c * nrow + r, default=empty
+                    )  # type:ignore[misc]
+                    for c in range(ncol)
+                ]
+                for r in range(nrow)
+            ],
+            info,
+        )
 
 
-def columnize(items, row_first=False, separator="  ", displaywidth=80, spread=False):
+def columnize(
+    items: List[str],
+    row_first: bool = False,
+    separator: str = "  ",
+    displaywidth: int = 80,
+    spread: bool = False,
+) -> str:
     """Transform a list of strings into a single string with columns.
 
     Parameters
@@ -743,7 +828,7 @@ def columnize(items, row_first=False, separator="  ", displaywidth=80, spread=Fa
     """
     warnings.warn(
         "`columnize` is Pending Deprecation since IPython 8.17."
-        "It is considered fro removal in in future version. "
+        "It is considered for removal in future versions. "
         "Please open an issue if you believe it should be kept.",
         stacklevel=2,
         category=PendingDeprecationWarning,
@@ -761,12 +846,14 @@ def columnize(items, row_first=False, separator="  ", displaywidth=80, spread=Fa
         separator = separator.ljust(int(info["optimal_separator_width"]))
     fmatrix: List[filter[int]] = [filter(None, x) for x in matrix]
     sjoin = lambda x: separator.join(
-        [y.ljust(w, " ") for y, w in zip(x, info["column_widths"])]
+        [y.ljust(w, " ") for y, w in zip(x, cast(List[int], info["column_widths"]))]
     )
     return "\n".join(map(sjoin, fmatrix)) + "\n"
 
 
-def get_text_list(list_, last_sep=' and ', sep=", ", wrap_item_with=""):
+def get_text_list(
+    list_: List[str], last_sep: str = " and ", sep: str = ", ", wrap_item_with: str = ""
+) -> str:
     """
     Return a string with a natural enumeration of items
 
